Package: rgeedim
Type: Package
Title: Search, Composite, and Download 'Google Earth Engine' Imagery
        with the 'Python' Module 'geedim'
Version: 0.4.0
Authors@R: c(person("Andrew", "Brown", role = c("aut", "cre"), email = "brown.andrewg@gmail.com"),
             person("Dugal", "Harris", role = "cph", comment = "'geedim' 'Python' module"))
Maintainer: Andrew Brown <brown.andrewg@gmail.com>
URL: https://humus.rocks/rgeedim/, https://github.com/brownag/rgeedim,
        https://geedim.readthedocs.io/
BugReports: https://github.com/brownag/rgeedim/issues
Repository: CRAN
Description: Search, composite, and download 'Google Earth Engine' imagery with 'reticulate' bindings for the 'Python' module 'geedim' by Dugal Harris. Read the 'geedim' documentation here: <https://geedim.readthedocs.io/>.
    Wrapper functions are provided to make it more convenient to use 'geedim' to download images larger than the 'Google Earth Engine' size limit <https://developers.google.com/earth-engine/apidocs/ee-image-getdownloadurl>.
    By default the "High Volume" API endpoint <https://developers.google.com/earth-engine/cloud/highvolume> is used to download data and this URL can be customized during initialization of the package.
SystemRequirements: Python (>= 3.6.0)
Config/reticulate: list( packages = list( list(package =
        "earthengine-api"), list(package = "geedim"), list(package =
        "google-auth") ) )
License: Apache License (>= 2)
Language: en-US
RoxygenNote: 7.3.3
Imports: utils, methods, reticulate
Suggests: terra, raster, tinytest, litedown
Depends: R (>= 3.5)
Encoding: UTF-8
VignetteBuilder: litedown
NeedsCompilation: no
Packaged: 2026-02-02 07:27:41 UTC; andrew
Author: Andrew Brown [aut, cre],
  Dugal Harris [cph] ('geedim' 'Python' module)
Date/Publication: 2026-02-02 07:50:02 UTC
Built: R 4.6.0; ; 2026-04-27 19:46:51 UTC; unix
